#!/usr/bin/env ruby

require_relative 'logging'

module PDO
  class SSHCmd
    include Logging

    def initialize(local, sshopts)

      @defaults = {
        :user => 'root',
        :port => '22',
        :ssh  => '/usr/bin/ssh',
        :sshopts  => {
          :ConnectTimeout         => '60',
          :StrictHostKeyChecking  => 'no',
        },
      }
      @local = local
      @sshopts = @defaults[:sshopts]
      @sshopts = @defaults[:sshopts].update sshopts if sshopts
      logger.debug { @sshopts.inspect }
    end
    
    def form(host, cmd)
      if host.match(/^(?:(\w+)@)?(\w+(?:\.\w+)*)(?::(\d+))?$/) then
        user, host, port = $1, $2, $3
      end

      cmd = cmd.dup # without dup, all tasks refer to the same
      cmd.strip!
      cmd.gsub! '_USER_', user ? user : @defaults[:user] 
      cmd.gsub! '_HOST_', host 
      cmd.gsub! '_PORT_', port ? port : @defaults[:port] 
      # when opts.cmd == '-', the command is read from stdin.
      # in this case multiple lines can be entered. here I'm doing some
      # simple substitution for "\n".
      cmd.gsub! "\n", '; '

      # escape "`$ characters in cmd, unless they're already escaped.
      if RUBY_VERSION.to_f < 1.9 then
        # v1.8 dose not support negative look-behind assertion, thus
        # doing it in 2 steps.
        if cmd.match /["`$]/ then
          if not cmd.match /['\\]["`$]/ then
            cmd.gsub! /(["`$])/, '\\\\\1'
          end
        end
      else
        cmd.gsub! /(?<!['\\])(["`$])/, '\\\\\1'
      end

      if @local then
        return cmd
      else
        ssh = @defaults[:ssh]
        ssh = [ ssh, '-l', "#{user}"].join ' ' if user
        ssh = [ ssh, '-p', "#{port}"].join ' ' if port
        @sshopts.each do |k, v| 
          ssh = [ssh, '-o', "#{k}=#{v}"].join ' '
        end
        ssh = [ ssh, "#{host}", "\"#{cmd}\""].join ' '
        return ssh
      end
    end
  end

  class Task
    include Logging

    def initialize
      @task_q = Queue.new
    end

    def add(task)
      @task_q << task
    end

    def next
      begin
        @task_q.deq(non_block=true)
      rescue ThreadError
        nil
      end
    end

    def size
      @task_q.length
    end

    def print_all
      tmp_q = Queue.new
      while not @task_q.empty? do
        task = @task_q.deq
        printf "%s\n", task
        tmp_q << task
      end
      @task_q = tmp_q
    end

  end # class SSHCmd
end # module PDO

# vim: set et ts=2 sts=2 sw=2 si sta :
